function [beta, zStar, pf,nFunctionCalls,flag] = ...
    HansoferLindND(gFunction,N,relBetaError,maxFunctionCalls)
% N-dimensional implementation of Hansofer-Lind Algorithm
if (nargin == 2)
    relBetaError = 0.01; % 1 percent error in beta accepted
    maxFunctionCalls = 10000;
end
if (nargin == 3)
    maxFunctionCalls = 10000;
end
betaOld = 0; % initialize reliability index
zBar = zeros(1,N); % initial guess
nFunctionCalls = 0; % keep track of samples
flag = 1; % flag = 1 means we have converged.
while (1) % to avoid infinite loops
    [g,grad_g] = gFunction(zBar);
    nFunctionCalls = nFunctionCalls+1+N; % 1 function call + N gradients 
    a0 = g - grad_g(:)'*zBar(:);
    gradNorm = norm(grad_g);
    betaNew = a0/gradNorm; % update based on linear approximation
    zBar = -grad_g*a0/gradNorm^2;
    if (abs(betaNew-betaOld)/betaNew < relBetaError) % we have converged 
        break;
    end
    if (nFunctionCalls > maxFunctionCalls) % to avoid infinite loop
        flag = 0; % we have not converged.
        break;
    end
    betaOld = betaNew;
end
beta = betaNew;
zStar = zBar;
pf = 1- Normal_Probability(0,1,beta);
