% Beam bending examples

example = 1;

if (example == 1) % deflection
    disp('Beam deflection failure');
    nSamples = 1e5;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    % Naive MC estimation
    g = gBeamDeflection([z1;z2]);
    Pf_MC = sum(g<0)/numel(g); % probability of failure
    
    % Failure space plotting
    indices = find(g<0);
    plot(z1(indices),z2(indices),'r.');
    axis([-1 4 -4 1]); axis equal;hold on;
    plot(0,0,'k*');
    grid on;
    
    %HL algorithm
    [beta, zStar, Pf_HL,nFunctionEvals,flag]=HansoferLind2D(@(zVec)gBeamDeflection(zVec));
   
    
    
    
    disp(['Pf_MC = ' num2str(Pf_MC)]);
    disp(['Pf_HL = ' num2str(Pf_HL)]);

    disp(['beta = ' num2str(beta)]);
    
    % Brietung correction
    [g,grad_z,H] =gBeamDeflection([zStar(1) zStar(2)]);
    
    A = zeros(3,3);
    A(1:2,1:2) = H;
    A(3,1:2) = grad_z';
    A(1:2,3) = grad_z;
    kappa = -det(A)/norm(grad_z)^3;
    Pf_Br = Pf_HL/sqrt(1+kappa*beta);
    disp(['Pf_Br = ' num2str(Pf_Br)]);

elseif (example == 2) % stress
    disp('Beam stress failure');
    nSamples = 1e5;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    % Naive Monte Carlo
    g = gBeamStress([z1;z2]);
    Pf_MC = sum(g<0)/numel(g); % probability of failure
    
    % Functional space plotting
    indices = find(g<0);
    plot(z1(indices),z2(indices),'r.');
    axis([-1 4 -4 1]); axis equal;hold on;
    plot(0,0,'k*');
    grid on;
    
     %HL algorithm
    [beta, zStar, Pf_HL,nFunctionEvals,flag]=HansoferLind2D(@(zVec)gBeamStress(zVec));
   

    disp(['Pf_MC = ' num2str(Pf_MC)]);
    disp(['Pf_HL = ' num2str(Pf_HL)]);
    
    
    disp(['beta = ' num2str(beta)]);
    
    % Brietung correction
    [g,grad_z,H] = gBeamStress([zStar(1) zStar(2)]);
    
    
    A = zeros(3,3);
    A(1:2,1:2) = H;
    A(3,1:2) = grad_z';
    A(1:2,3) = grad_z;
    kappa = -det(A)/norm(grad_z)^3;
    Pf_Br = Pf_HL/sqrt(1+kappa*beta);
    disp(['Pf_Br = ' num2str(Pf_Br)]);
   
end
