function [g, gradZ] = gLBracketDeflectionWithGradient(z,h)
if (nargin == 1)
    h = 0.001; % step size for finite difference
end
g = gLBracketDeflection(z);
N = numel(z);
gradZ = zeros(1,N);
if (N ~= 3)
    disp('Size of z must be equal to 3');
    return;
end
if (nargout == 1)
    return; % no need to compute gradient
end
z0 = z; % save the value
for i = 1:N
    z = z0;
    z(i) = z(i) + h; % increment the i-th value of z
    gh = gLBracketDeflection(z);
    gradZ(i) = (gh-g)/h;
end
end
function [g] = gLBracketDeflection(zVec)
% return g function
LNominal = 0.1;
LStdDev = 0.003;
HNominal = 0.04;
HStdDev = 0.002;
RNominal = 0.02; % fillet radius
RStdDev = 0.002;
L = LNominal + zVec(1)*LStdDev;
H = HNominal + zVec(2)*HStdDev;
R = RNominal + zVec(3)*RStdDev;
designVariables = [L H R];
fem = LBracket(designVariables);
fem = fem.assemble();
fem = fem.solve();
deltaNominal = 5.6433e-05;
deltaAllowable = 2*deltaNominal;
g = deltaAllowable -fem.myMaxDelta;
end
function fem = LBracket(designVariables)
% designVariables: L, H and R
if (nargin == 0)
    designVariables = [0.1 0.04 0.02];
end
% create LBracket with given fillet radius
L = designVariables(1);%length and width in meters
H = designVariables(2);%
R = designVariables(3);  % fillet radius
d = 0.005; % for force application
LBracket.vertices = [0 0;L 0; L H; L-d H; H+R H; H H+R; H L; 0 L; H+R H+R]';
LBracket.edges = [1 2; 2 3; 3 4; 4 5; 5 6; 6 7; 7 8; 8 1];
LBracket.arcs = [5 9]';
fem = TriElasticity(LBracket,3000);
fem = fem.setYoungsModulus(2.1e11);
fem = fem.setPoissonsRatio(0.28);
fem = fem.fixEdge(7);
fem = fem.applyYForceOnEdge(3,-100000);
end

