
close all;

example = 3;

if ((example == 1) || (example == 2))
    nSamples = 1e6;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);

    g =gNonlinearEx6([z1;z2]);
    indices = find(g<0);
    plot(z1(indices),z2(indices),'r.');
    axis equal;hold on;
    plot(0,0,'k*');
    grid on;
    
    
    Pf_MC = sum(g<0)/numel(g); % probability of failure

    [beta, zStar, Pf_HL,nFunctionEvals,flag]=HansoferLind2D(@(zVec)gNonlinearEx6(zVec));
   
    [g,grad_z,H] =gNonlinearEx6([zStar(1) zStar(2)]);
    
    A = zeros(3,3);
    A(1:2,1:2) = H;
    A(3,1:2) = grad_z';
    A(1:2,3) = grad_z;
    kappa = - det(A)/norm(grad_z)^3;
    Pf_Br = Pf_HL/sqrt(1+kappa*beta);
    
    disp(['Pf_MC = ' num2str(Pf_MC)]);
    disp(['Pf_HL = ' num2str(Pf_HL)]);
    disp(['Pf_Br = ' num2str(Pf_Br)]);
end
if (example == 2)
    % we use the rectangle with rounded corners to plot a circle 
    rectangle('Position',[-beta -beta 2*beta 2*beta],'Curvature',[1 1],'FaceColor','g','EdgeColor','g') 
    grid on;axis equal;
    plot(zStar(1),zStar(2),'r*'); hold on;
    plot(0,0,'db');
    
elseif (example == 3) 
    % use HL algorithm to find out beta, then use Selective Evaluation
    [beta, zStar, Pf_HL,nFunctionEvals,flag]=HansoferLind2D(@(zVec)gNonlinearEx6(zVec));
    % create samples as before.
    nSamples = 1e6;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    % Only consider samples whose distance is greater than or equal to beta
    d = sqrt(z1.^2 + z2.^2);
    indices  = find(d >= beta);

    plot(z1(indices),z2(indices),'b.');
    axis equal;hold on;
    plot(0,0,'k*');
    grid on;

    % Now evaluate the g-function only on the selected indices
    g =gNonlinearEx6([z1(indices);z2(indices)]);
    Pf_SE = sum(g<0)/nSamples; % probability of failure
    disp(['Pf_SE = ' num2str(Pf_SE)]);
    disp(['nFunctionEvals = ' num2str(length(indices))]);

end
